/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>

#include "pia.h"
#include "keyboard.h"
#include "snapshot.h"
#include "logging.h"

static int control = 0;

static int matrix[9][9];

static int keymap[128][2] = {
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 0 - 7 */
	{5,5}, {6,5}, {8,8}, {8,8}, {1,6}, {0,6}, {8,8}, {8,8}, /* 8 - 15 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 16 - 23 */
	{8,8}, {8,8}, {8,8}, {2,6}, {8,8}, {8,8}, {8,8}, {8,8}, /* 24 - 31 */
	{7,5}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 32 - 39 */
	{8,8}, {8,8}, {8,8}, {8,8}, {4,1}, {2,1}, {6,1}, {7,1}, /* 40 - 47 */
	{0,0}, {1,0}, {2,0}, {3,0}, {4,0}, {5,0}, {6,0}, {7,0}, /* 48 - 55 */
	{0,1}, {1,1}, {8,8}, {3,1}, {8,8}, {5,1}, {8,8}, {8,8}, /* 56 - 63 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 64 - 71 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 72 - 79 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 80 - 87 */
	{8,8}, {8,8}, {8,8}, {0,2}, {2,6}, {8,8}, {5,5}, {8,8}, /* 88 - 95 */
	{1,6}, {1,2}, {2,2}, {3,2}, {4,2}, {5,2}, {6,2}, {7,2}, /* 96 - 103 */
	{0,3}, {1,3}, {2,3}, {3,3}, {4,3}, {5,3}, {6,3}, {7,3}, /* 104 - 111 */
	{0,4}, {1,4}, {2,4}, {3,4}, {4,4}, {5,4}, {6,4}, {7,4}, /* 112 - 119 */
	{0,5}, {1,5}, {2,5}, {8,8}, {8,8}, {8,8}, {8,8}, {1,6}, /* 120 - 127 */
};

void keyboard_init(void) {
	/* Register PIA devices */
	PIA_0A.wrfunc[0] = keyboard_row_write;
	PIA_0B.wrfunc[0] = keyboard_column_write;
}

void keyboard_reset(void) {
	memset(matrix, 0, sizeof(matrix));
}

void keyboard_poll(void) {
	SDL_Event event;
	SDLKey sym;
	while (SDL_PollEvent(&event) == 1) {
		switch(event.type) {
			case SDL_KEYDOWN:
				sym = event.key.keysym.sym;
				if (sym == SDLK_LCTRL || sym == SDLK_RCTRL) {
					control = 1;
					break;
				}
				if (sym == SDLK_c && control) {
					exit(0);
				}
				if (sym == SDLK_s && control) {
					snapshot_state = 1;
					break;
				}
				if (sym == SDLK_l && control) {
					snapshot_state = 2;
					break;
				}
				if (sym == SDLK_LSHIFT || sym == SDLK_RSHIFT) {
					matrix[7][6] = 1;
				} else {
					if (sym < 128) {
						matrix[keymap[sym][0]][keymap[sym][1]] = 1;
					}
				}
				break;
			case SDL_KEYUP:
				sym = event.key.keysym.sym;
				if (sym == SDLK_LCTRL || sym == SDLK_RCTRL) {
					control = 0;
					break;
				}
				if (sym == SDLK_LSHIFT || sym == SDLK_RSHIFT) {
					matrix[7][6] = 0;
				} else {
					if (sym < 128) {
						matrix[keymap[sym][0]][keymap[sym][1]] = 0;
					}
				}
				break;
			default:
				break;
		}
	}
}

void keyboard_column_write(Uint8 mask) {
	Uint8 row = 0xff;
	int i, j;
	for (i = 0; i < 8; i++) {
		if ((mask & (1 << i)) == 0) {
			for (j = 0; j < 7; j++) {
				if (matrix[i][j]) {
					row &= ~(1 << j);
				}
			}
		}
	}
	pia_data_set(&PIA_0A, row);
}

void keyboard_row_write(Uint8 mask) {
	Uint8 col = 0xff;
	int i, j;
	for (i = 0; i < 7; i++) {
		if ((mask & (1 << i)) == 0) {
			for (j = 0; j < 8; j++) {
				if (matrix[j][i]) {
					col &= ~(1 << j);
				}
			}
		}
	}
	pia_data_set(&PIA_0B, col);
}
