/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  See LICENSE for redistribution conditions. */

#ifndef __PIA_H__
#define __PIA_H__

#include <SDL/SDL_types.h>

#define MAX_WRFUNCS 4

typedef void (*write_function)(Uint8 byte);
typedef void (*interrupt_function)(void);
typedef struct {
	Uint8 control_register;
	int interrupt_enable;
	int interrupt_transition;
	int register_select;  /* 0 = DDR, SET = OR */
	Uint8 direction_register;
	Uint8 output_register;
	/* ignore Cx2 stuff for now */
	write_function wrfunc[MAX_WRFUNCS];
	interrupt_function intfunc;
} pia_port;

extern pia_port PIA_0A, PIA_0B, PIA_1A, PIA_1B;

void pia_init(void);
void pia_control_write(pia_port *p, Uint8 byte);
Uint8 pia_control_read(pia_port *p);
/* This is for MPU writes to the PIA */
void pia_data_write(pia_port *p, Uint8 byte);
Uint8 pia_data_read(pia_port *p);
/* Generate an interrupt */
void pia_set_ca1(pia_port *p);
/* This is called by devices to set input bits */
void pia_data_set(pia_port *p, Uint8 byte);

#endif  /* __PIA_H__ */
